-------------------------------------------------------------------------------
-- mental ray .map Converter and Collector
-- by David Baker (dave@maxplugins.de)
-- Based on Neil Blevins MapCollector.ms
-- v 1.10
-- Created On: 15/04/07
-- Updated On: 07/05/07 - Added UNC option
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Requirements:
-- Most importantly, you need the program imf_copy.exe
--
-- You also need to make a change to your base.mi
-- (in maxfolder/mentalray/shaders_standard/include).
-- Search for the text 'gui "gui_mib_texture_lookup2" {'
-- and then put a hash (#) in front of the word 'hidden'
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Description:
-- Lets you collect all the bitmaps from your scene, convert them to mental ray
-- .map files, place them into a directory, and replace all bitmap textures with
-- a 'mib_texture_lookup2' shader containing the new .map files.
-- If the backup option is checked, the script automatically saves a backup of
-- your file (filename_backup.max), and when it has finished converting the bitmaps,
-- automatically saves a new copy of the file (filename__mr_maps.max)
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
(
--global start_mapc
--global mapc_rollout
global mapc_floater

local get_all_medit_mats,get_all_mats_in_selected_objs,get_maps_from_mat,get_all_materials,get_sub_materials,get_all_scene_materials
local name_compare_case_insensitive,trim_dups,get_bitmaps_from_mats,file_exist,lowercase,add_slash,check_saveFile,backup_scene,save_map_scene
local save_ini,load_ini,mapc_out_dir,imf_out_dir
local map_ini_file = getDir #plugcfg + "\\mapconverter.ini"

-- Variables
namefile = GetFileNameFile MaxFilename
i1_value = true
i2_value = false
i8_value = false
copy_move_value = 1
update_mats_value = true
overwrite_value = false
backup_value = true
unc_value = false

-- Functions

fn save_ini =
 	(
	setINISetting map_ini_file "Settings" "MAP_Path" (mapc_out_dir as string)
	setINISetting map_ini_file "Settings" "IMF_Copy_Path" (imf_out_dir as string)
 	)

fn load_ini =
 	(
 	if doesFileExist map_ini_file then
		(
		mapc_out_dir = (getINISetting map_ini_file "Settings" "MAP_Path")
		imf_out_dir = (getINISetting map_ini_file "Settings" "IMF_Copy_Path")
		)
	else
		(
		mapc_out_dir = undefined
		imf_out_dir = undefined
		)
	)

fn backup_scene =
	(
	if namefile != "" then
		(
		current_scene = (namefile + "_backup.max")
		check_saveFile current_scene
		)
	else namefile = "maptest"
	)

fn save_map_scene =
	(
	if namefile != "" then
		(
		current_scene = (namefile + "_mr_maps.max")
		check_saveFile current_scene
		)
	else namefile = "maptest"
	)

fn check_saveFile filename =
	(
	if getFilenamePath filename == "" do filename = getdir #scene + "\\" + filename
	if (not (doesfileexist filename)) or querybox ("filename: "+filename + "\nexists. Overwrite?") then
		(
		saveMaxFile filename
		true
		)
	else
		false
	)

fn lowercase instring =
	(
	upper="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	lower="abcdefghijklmnopqrstuvwxyz" 
	outstring=copy instring
	for i = 1 to outstring.count do
		(
		j=findString upper outstring[i]
		if (j != undefined) do outstring[i]=lower[j]
		)
	outstring
	)

fn file_exist fname =
	(
	(getfiles fname).count != 0
	)

fn trim_dups a =
	(
	for i in a.count to 1 by -1 do
		(
		idx = findItem a a[i]
		if (idx != 0) AND (idx != i) do deleteItem a i
		)
	a
	)

fn get_all_medit_mats =
	(
	b = #()
	for i = 1 to 24 do
		(
		for i in (get_all_materials meditmaterials[i]) do
			(
			if (finditem b i) == 0 then append b i
			)
		)
	trim_dups b
	qsort b name_compare_case_insensitive
	return b
	)

fn get_all_mats_in_selected_objs =
	(
	b = #()
	for i in selection do
		(
		if i.material != undefined then
			(
			for i in (get_all_materials i.material) do
				(
				if (finditem b i) == 0 then append b i
				)
			)
		)
	trim_dups b
	qsort b name_compare_case_insensitive
	return b
	)

fn get_maps_from_mat mat x=
	(
	for i = 1 to (getnumsubtexmaps mat) do
		(
		if superclassof (getsubtexmap mat i) == textureMap then
			(
			if getnumsubtexmaps (getsubtexmap mat i) != 0 then
				(
				get_maps_from_mat (getsubtexmap mat i) x
				)
			else
				(
				if classof (getsubtexmap mat i) == Bitmaptexture then
					(
					if update_mats_value == true then
						(
						old_bm = (getsubtexmap mat i).filename
						setsubtexmap mat i (mib_texture_lookup2())
						(getsubtexmap mat i).factor = 1
						(getsubtexmap mat i).tex = old_bm as string
						append x (getsubtexmap mat i)
						)
					else
						(
						append x (getsubtexmap mat i)
						)
					)
				)
			)
		)
	)

fn get_bitmaps_from_mats a =
	(
	b = #()
	for i in a do
		(
		get_maps_from_mat i b
		)
	trim_dups b
	qsort b name_compare_case_insensitive
	return b
	)

fn get_all_materials mat =
	(
	aa = #()
	x = #()
	append aa mat
	get_sub_materials mat x
	for i in x do
		(
		if finditem aa i == 0 then append aa i
		)
	return aa
	)

fn get_sub_materials mat x =
	(
	if mat.numsubs != 0 then
		(
		for i = 1 to mat.numsubs do
			(
			if mat[i] != undefined then
				(
				if iskindof mat[i] material == true then
					(
					if classof mat[i] == SubAnim then
						(
						if superclassof mat[i].object == material then
							(
							if finditem x (mat[i].object) == 0 then append x (mat[i].object)
							)
						)
					if superclassof mat[i] == material then
						(
						if finditem x (mat[i]) == 0 then append x (mat[i])
						)
					)
				get_sub_materials mat[i] x
				)
			)
		)
	)

fn get_all_scene_materials =
	(
	a = #()
	for i in $* do
		(
		if i.material != undefined then
			(
			for i in (get_all_materials i.material) do
				(
				if (finditem a i) == 0 then append a i
				)
			)
		)
	qsort a name_compare_case_insensitive
	return a
	)

fn name_compare_case_insensitive s1 s2 =
	(
	if (lowercase s1.name) < (lowercase s2.name) then return -1
	else if (lowercase s1.name) > (lowercase s2.name) then return 1
	else return 0
	)

fn add_slash t1 =
	(
	fix=""
	old=filterstring t1 "\\"
	for i=1 to (old.count-1) do fix+=(old[i]+"\\\\")
	fix+=(old[old.count])
	fix
	)

fn start_mapc =
	(
	if backup_value == true do backup_scene()
--	mapc_out_dir2 = add_slash mapc_out_dir

	imf_out_dir2 = add_slash imf_out_dir
	imf_out_drive = substring imf_out_dir 1 2

	print "Collecting bitmap info"

	a = #()
	b = #()
	c = #()
	all_maps = #()
	all_bm = #()
	total_files_copied = 0

	if i1_value == true then a = get_all_scene_materials()
	if i2_value == true then b = get_all_medit_mats()
	if i8_value == true then c = get_all_mats_in_selected_objs()

	all_mats = a + b + c
	all_bm = get_bitmaps_from_mats all_mats

	trim_dups all_bm

	print "Bitmap info collected, converting bitmaps."

	progressStart "Map Converter"
	escapeEnable = false

	for i = 1 to all_bm.count do
		(
		m = ((i as float)/(all_bm.count as float))*100

		if update_mats_value == true then
		  (
		  infile = all_bm[i].tex
		  )
		else
		  (
		  infile = all_bm[i].filename
		  )
		infile2 = add_slash infile
		outfile = (mapc_out_dir + "\\" + (getFilenameFile infile) + ".map")
		outfile2 = add_slash outfile
		imf_command = ("/S /C \"" + imf_out_drive + " && cd \"" + imf_out_dir2 + "\" && imf_copy -p -r \"" + infile2 + "\" \"" + outfile2 + "\" map\"")

		if overwrite_value == true then
			(
			ShellLaunch "cmd.exe" imf_command
			total_files_copied += 1
			)
		else
			(
			if file_exist outfile2 == false then
				(
				answer = ShellLaunch "cmd.exe" imf_command
				if answer != false then total_files_copied += 1
				)
			)

--		if update_mats_value == true then all_bm[i].tex = outfile
		if update_mats_value == true then
			(
			if unc_value == true then
				(
				uncout = pathConfig.convertPathToUnc outfile
				all_bm[i].tex = uncout
				)
			else
				(
				all_bm[i].tex = outfile
				)
			)
		progressUpdate m
		)

	progressEnd()
	if backup_value == true do save_map_scene()
	print "Map Converter Results"
	print ("Bitmaps Converted and Copied: " + (total_files_copied as string))
	)

-- The Script

rollout mapc_rollout ".Map Converter and Collector"
	(
	group "Path To Collect Maps To"
	(
--	checkbox same_dir "Use Original Bitmap Location for .Map Files" checked:same_dir_value
	button mapc_getpath "Path" align:#left across:2
	edittext mapc_pathlabel "" fieldWidth:158 offset:[-60,4] text:(mapc_out_dir as string)
	)

	on mapc_getpath pressed do
		(
		mapc_out_dir = getSavePath()
		mapc_pathlabel.text = (mapc_out_dir as string)
		)
	on mapc_pathlabel entered text do
		(
		if mapc_pathlabel.text == "undefined" then mapc_out_dir = undefined
		else mapc_out_dir = mapc_pathlabel.text
		)

	group "Path To imf_copy.exe"
	(
	button imf_getpath "Path" align:#left across:2
	edittext imf_pathlabel "" fieldWidth:158 offset:[-60,4] text:(imf_out_dir as string)
	)

	on imf_getpath pressed do
		(
		imf_out_dir = getSavePath()
		imf_pathlabel.text = (imf_out_dir as string)
		)
	on imf_pathlabel entered text do
		(
		if imf_pathlabel.text == "undefined" then imf_out_dir = undefined
		else imf_out_dir = imf_pathlabel.text
		)

	group "Include"
	(
	checkbox i1 "Maps In Scene Materials" checked:i1_value
	checkbox i2 "Material Editor" checked:i2_value
	checkbox i8 "Maps Applied to Selected Objects" checked:i8_value
	)

	on i1 changed state do (if i1.checked == true then i1_value = true else i1_value = false)
	on i2 changed state do (if i2.checked == true then i2_value = true else i2_value = false)
	on i8 changed state do (if i8.checked == true then i8_value = true else i8_value = false)

	group "Options"
	(
	checkbox update_mats "Update Scene" checked:update_mats_value
	checkbox overwrite_checkbox "Overwrite Existing Bitmaps" checked:overwrite_value
	checkbox backup_checkbox "Backup Scene Before Conversion" checked:backup_value
	checkbox unc_checkbox "Convert Output Path to UNC" checked:unc_value
	)


	on update_mats changed state do (if update_mats.checked == true then update_mats_value = true else update_mats_value = false)
	on overwrite_checkbox changed state do (if overwrite_checkbox.checked == true then overwrite_value = true else overwrite_value = false)
	on backup_checkbox changed state do (if backup_checkbox.checked == true then backup_value = true else backup_value = false)
	on unc_checkbox changed state do (if unc_checkbox.checked == true then unc_value = true else unc_value = false)

	button domapcollection "Convert Maps" width:200 align:#center toolTip:"Convert Maps"
	on domapcollection pressed do
		(
		if mapc_out_dir != undefined then
			(
			if imf_out_dir != undefined then
				(
				start_mapc()
				)
			else (MessageBox "Please pick a valid path to imf_copy.exe" title:"Map Converter")
			)
		else (MessageBox "Please pick a valid output path" title:"Map Converter")
		)
 	on mapc_rollout close do save_ini()
	)

rollout ab "About"
	(
	Group ""
	  (
	  Label a1 "mental ray .MAP Converter v1.1"
	  Label a2 "written by David Baker"
	  Label a3 "April / May 2007"
	  Hyperlink a4 "E-mail: dave@maxplugins.de" address:"mailto:dave@maxplugins.de" align:#center
	  Hyperlink a5 "http://www.maxplugins.de/" address:"http://www.maxplugins.de/" align:#center
	  )
	)

try(closeRolloutFloater mapc_floater)catch()
--if mapc_floater != undefined then CloseRolloutFloater mapc_floater
mapc_floater = newRolloutFloater "MapConverter v1.1" 260 380
load_ini()
addRollout mapc_rollout mapc_floater
addRollout ab mapc_floater rolledup:true
)
-------------------------------------------------------------------------------